@@ -1,58 +1,86 @@
 // -*- C++ -*-
-/* This file is part of
- * ======================================================
+/**
+ * \file paragraph_pimpl.h
+ * This file is part of LyX, the document processor.
+ * Licence details can be found in the file COPYING.
  *
- *           LyX, The Document Processor
+ * \author Lars Gullik Bjnnes
+ * \author Jean-Marc Lasgouttes
+ * \author John Levon
+ * \author Andr Pnitz
  *
- *	    Copyright 1995 Matthias Ettrich
- *          Copyright 1995-2001 The LyX Team.
- *
- * ====================================================== */
+ * Full author contact details are available in file CREDITS.
+ */
 
 #ifndef PARAGRAPH_PIMPL_H
 #define PARAGRAPH_PIMPL_H
 
 #include "paragraph.h"
+
+#include "changes.h"
+#include "lyxfont.h"
 #include "ParagraphParameters.h"
-#include "counters.h"
+
+#include <boost/scoped_ptr.hpp>
 
 class LyXLayout;
 
-struct Paragraph::Pimpl {
-	///
-	typedef std::vector<value_type> TextContainer;
 
+class Paragraph::Pimpl {
+public:
 	///
 	Pimpl(Paragraph * owner);
-	/// Copy constructor
-	Pimpl(Pimpl const &, Paragraph * owner, bool same_ids = false);
+	/// "Copy constructor"
+	Pimpl(Pimpl const &, Paragraph * owner);
 	///
-	lyx::pos_type size() const {
-		return text.size();
-	}
-	///
-	bool empty() const {
-		return text.empty();
-	}
-	///
-	void clear();
-	///
-	void setContentsFromPar(Paragraph const * par);
+	void setContentsFromPar(Paragraph const & par);
+
+	//
+	// Change tracking
+	//
+	/// set tracking mode
+	void trackChanges(Change::Type type = Change::UNCHANGED);
+	/// stop tracking
+	void untrackChanges();
+	/// set all text as new for change mode
+	void cleanChanges();
+	/// look up change type at given pos
+	Change::Type lookupChange(lyx::pos_type pos) const;
+	/// look up change at given pos
+	Change const lookupChangeFull(lyx::pos_type pos) const;
+	/// is there a change in the given range ?
+	bool isChanged(lyx::pos_type start, lyx::pos_type end) const;
+	/// is there a non-addition in this range ?
+	bool isChangeEdited(lyx::pos_type start, lyx::pos_type end) const;
+	/// set change at pos
+	void setChange(lyx::pos_type pos, Change::Type type);
+	/// set full change at pos
+	void setChangeFull(lyx::pos_type pos, Change change);
+	/// mark as erased
+	void markErased(bool);
+	/// accept change
+	void acceptChange(lyx::pos_type start, lyx::pos_type end);
+	/// reject change
+	void rejectChange(lyx::pos_type start, lyx::pos_type end);
+	/// are we tracking changes ?
+	bool tracking() const { return changes_.get(); }
+
 	///
 	value_type getChar(lyx::pos_type pos) const;
 	///
 	void setChar(lyx::pos_type pos, value_type c);
 	///
-	void insertChar(lyx::pos_type pos, value_type c, LyXFont const & font);
+	void insertChar(lyx::pos_type pos, value_type c, Change change);
 	///
-	void insertInset(lyx::pos_type pos, Inset * inset, LyXFont const & font);
+	void insertInset(lyx::pos_type pos, InsetBase * inset, Change change);
+	/// definite erase
+	void eraseIntern(lyx::pos_type pos);
+	/// erase the given position. Returns true if it was actually erased
+	bool erase(lyx::pos_type pos);
+	/// erase the given range
+	int erase(lyx::pos_type start, lyx::pos_type end);
 	///
-	void erase(lyx::pos_type pos);
-	///
-	LyXFont const realizeFont(LyXFont const & font,
-				  BufferParams const & bparams) const;
-	///
-	Inset * inset_owner;
+	InsetBase * inset_owner;
 
 	/** A font entry covers a range of positions. Notice that the
 	    entries in the list are inserted in random order.
@@ -65,21 +93,29 @@ struct Paragraph::Pimpl {
 	    and font_i covers the chars in positions pos_{i-1}+1,...,pos_i
 	    (font_1 covers the chars 0,...,pos_1) (Dekel)
 	*/
-	struct FontTable  {
+	class FontTable  {
+	public:
 		///
 		FontTable(lyx::pos_type p, LyXFont const & f)
-			: pos_(p)
-			{
-				font_ = container.get(f);
-			}
+			: pos_(p), font_(f)
+		{
+#if 0
+			font_ = container.get(f);
+#endif
+		}
 		///
 		lyx::pos_type pos() const { return pos_; }
 		///
 		void pos(lyx::pos_type p) { pos_ = p; }
 		///
-		LyXFont const & font() const { return *font_; }
+		LyXFont const & font() const { return font_; }
+#if 0
 		///
 		void font(LyXFont const & f) { font_ = container.get(f);}
+#else
+		///
+		void font(LyXFont const & f) { font_ = f;}
+#endif
 	private:
 		/// End position of paragraph this font attribute covers
 		lyx::pos_type pos_;
@@ -92,16 +128,20 @@ struct Paragraph::Pimpl {
 		    The values LyXFont::IGNORE_* and LyXFont::TOGGLE are NOT
 		    allowed in these font tables.
 		*/
+#if 0
 		boost::shared_ptr<LyXFont> font_;
 		///
 		static ShareContainer<LyXFont> container;
+#else
+		LyXFont font_;
+#endif
 	};
 	///
-	friend struct matchFT;
+	friend class matchFT;
 	///
-	struct matchFT {
+	class matchFT {
+	public:
 		/// used by lower_bound and upper_bound
-		inline
 		int operator()(FontTable const & a, FontTable const & b) const {
 			return a.pos() < b.pos();
 		}
@@ -111,9 +151,7 @@ struct Paragraph::Pimpl {
 	typedef std::vector<FontTable> FontList;
 	///
 	FontList fontlist;
-	///
-	Paragraph * TeXDeeper(Buffer const *, BufferParams const &,
-				 std::ostream &, TexRow & texrow);
+
 	///
 	void simpleTeXBlanks(std::ostream &, TexRow & texrow,
 			     lyx::pos_type const i,
@@ -121,11 +159,14 @@ struct Paragraph::Pimpl {
 			     LyXFont const & font,
 			     LyXLayout const & style);
 	///
-	void simpleTeXSpecialChars(Buffer const *, BufferParams const &,
+	void simpleTeXSpecialChars(Buffer const &, BufferParams const &,
 				   std::ostream &, TexRow & texrow,
-				   bool moving_arg,
+				   OutputParams const &,
 				   LyXFont & font, LyXFont & running_font,
-				   LyXFont & basefont, bool & open_font,
+				   LyXFont & basefont,
+				   LyXFont const & outerfont,
+				   bool & open_font,
+				   Change::Type & running_change,
 				   LyXLayout const & style,
 				   lyx::pos_type & i,
 				   unsigned int & column, value_type const c);
@@ -134,8 +175,6 @@ struct Paragraph::Pimpl {
 	void validate(LaTeXFeatures & features,
 		      LyXLayout const & layout) const;
 
-	///
-	Paragraph * getParFromID(int id) const;
 	///
 	unsigned int id_;
 	///
@@ -144,13 +183,16 @@ struct Paragraph::Pimpl {
 	ParagraphParameters params;
 
 private:
+	///
+	lyx::pos_type size() const { return owner_->size(); }
 	/// match a string against a particular point in the paragraph
-	bool isTextAt(string const & str, lyx::pos_type pos) const;
+	bool isTextAt(std::string const & str, lyx::pos_type pos) const;
+
+	/// for recording and looking up changes in revision tracking mode
+	boost::scoped_ptr<Changes> changes_;
 
 	/// Who owns us?
 	Paragraph * owner_;
-	///
-	TextContainer text;
 };
 
 #endif
